#!/usr/bin/python3.5
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: config
#------------------------------------------------------------------------------
# Class grouping the configuration data 
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

from lxml import etree
from datetime import datetime
import os


#==============================================================================
# Parameters
#==============================================================================

# Base Ontology Référence
BASE_ONTOLOGY_REF = ['base', 'base-ontology']



#==============================================================================
# Class
#==============================================================================

class Config:
    """Configuration data for Tenet process """
    
    #-------------------------------------------------
    # Constructor
    #-------------------------------------------------
    
    def __init__(self, 
                 config_file, 
                 uuid_str, source_corpus, target_ref='base',
                 base_dir=None,
                 base_output_dir=None,
                 technical_dir_path=None):
        
        # -- Config XML Tree
        self.config_tree = etree.parse(config_file)
        
        # -- Base paremeters
        self.config_file = config_file
        c_base = self.config_tree.xpath("base")[0]
        self.uuid_str = uuid_str
        self.technical_base_name = uuid_str
        self.source_corpus = source_corpus
        self.target_ref = target_ref
        self._process_level = c_base.get("process_level")
        self.source_type = c_base.get("source_type")
        self.extraction_scheme = c_base.get("extraction_scheme")
        
        # # -- CTS Reference
        # self.cts_ref = ""

        # -- Directories
        c_dir = self.config_tree.xpath("directory")[0]
        if base_dir == None:
            self.base_dir = c_dir.get("base_dir")
        else:
            self.base_dir = base_dir
        self.structure_dir = self.base_dir + c_dir.get("structure")
        self.cts_dir = self.base_dir + c_dir.get("cts")
        self._target_frame_dir = self.base_dir + c_dir.get("target_frame")
        self.input_doc_dir = ''
        if base_output_dir == None:
            self.base_output_dir = self.source_corpus
        else:
            self.base_output_dir = base_output_dir
        self.output_dir = ''
        self.technical_dir_path = technical_dir_path
        self.sentence_output_dir = ''
        
        # -- Config File Definition
        c_file = self.config_tree.xpath("file")[0]
        self.schema_file = ""
        self.semantic_net_file = self.structure_dir
        self.semantic_net_file += c_file.get("semantic_net_schema") + ".ttl"
        self.config_param_file = self.structure_dir
        self.config_param_file += c_file.get("config_param") + ".ttl"
        self.base_ontology_file = self.structure_dir
        self.base_ontology_file += 'base-ontology.ttl'
        self.cts_file = f"{self.cts_dir}{self.extraction_scheme}.py"
        # self.cts_file = ""
        
        # -- Ontology References
        c_ref = self.config_tree.xpath("reference")[0]
        self.base_uri = c_ref.get("default_base_uri")
        self.onto_suffix = c_ref.get("default_ontology_suffix")
        self.onto_seed_suffix = c_ref.get("default_ontology_seed_suffix")
        
        # -- Source File Definition
        self.source_sentence_file = '**/*'
        
        # -- Target File Definition
        target_ontology = self.target_frame_dir + self.target_ref
        self.frame_ontology_file = target_ontology + self.onto_suffix
        self.frame_ontology_seed_file = target_ontology + self.onto_seed_suffix
        
        # -- Output
        c_out = self.config_tree.xpath("output")[0]
        self.output_ontology_namespace = c_out.get("ontology_namespace")
        # self.output_file = self.sentence_output_dir + self.uuid_str + ".ttl"
        self.output_file = self.sentence_output_dir + self.technical_base_name + ".ttl"
            
        
    #-------------------------------------------------
    # Accessors
    #-------------------------------------------------
    
    # -- Property: technical_base_name
    
    @property
    def technical_base_name(self):
        return self._technical_base_name
    
    @technical_base_name.setter
    def technical_base_name(self, new_value):
        new_value = new_value.replace('https', '')
        new_value = new_value.replace('http', '')
        new_value = new_value.replace(':', '')
        new_value = new_value.replace('/', '_')
        if new_value[-1] == '_': new_value = new_value[:-1]
        if new_value.startswith('_'): new_value = new_value[1:]
        if new_value.startswith('_'): new_value = new_value[1:]
        if len(new_value) == 0: new_value = 'defaultBaseName'
        self._technical_base_name = new_value
        
    
    # -- Property: process_level
    
    def _get_process_level(self):
        return self._process_level
    
    def _set_process_level(self, new_level):
        self._process_level = new_level
        
    process_level = property(_get_process_level, _set_process_level)
        
    
    # -- Property: target_dir
    
    def _get_target_frame_dir(self):
        return self._target_frame_dir
    
    def _set_target_frame_dir(self, new_target_frame_dir):
        self._target_frame_dir = self.base_dir + new_target_frame_dir
        target_ontology = self.target_frame_dir + self.target_ref
        self.frame_ontology_file = target_ontology + self.onto_suffix
        self.frame_ontology_seed_file = target_ontology + self.onto_seed_suffix
    
    target_frame_dir = property(_get_target_frame_dir, _set_target_frame_dir)
    
    
    # -- Property: base_output_dir
    
    def _get_base_output_dir(self):
        return self._base_output_dir
    
    def _set_base_output_dir(self, base_output_dir=''):
        self._base_output_dir = base_output_dir
    
    base_output_dir = property(_get_base_output_dir, _set_base_output_dir)
    
    
    # -- Property: output_dir
    
    def _get_output_dir(self):
        return self._output_dir
    
    def _set_output_dir(self, output_dir_complement=''):
        c_dir = self.config_tree.xpath("directory")[0]
        self._output_dir = self.base_output_dir
        # self._output_dir += self.uuid_str + output_dir_complement
        self._output_dir += self.technical_base_name + output_dir_complement
        self._output_dir += '-' + datetime.now().strftime('%Y%m%d')
        self._output_dir += '/'
    
    output_dir = property(_get_output_dir, _set_output_dir)
    
    
    # -- Property: sentence_output_dir
    
    def _get_sentence_output_dir(self):
        return self._sentence_output_dir
    
    def _set_sentence_output_dir(self, sentence_output_dir_complement=''):
        if self.technical_dir_path is not None:
            self._sentence_output_dir = self.technical_dir_path
        else:
            self._sentence_output_dir = os.path.dirname(self.source_corpus) + '/'
        if sentence_output_dir_complement != '':
            self._sentence_output_dir += self.technical_base_name # self.uuid_str
            self._sentence_output_dir += sentence_output_dir_complement
            self._sentence_output_dir += '/'
        # self.output_file = self.sentence_output_dir + self.uuid_str + ".ttl"
        self.output_file = self.sentence_output_dir + self.technical_base_name + ".ttl"
    
    sentence_output_dir = property(_get_sentence_output_dir, _set_sentence_output_dir)
        
    
    # -- Property: input_doc_dir
    
    def _get_input_doc_dir(self):
        return self._input_doc_dir
    
    def _set_input_doc_dir(self, output_dir_complement):
        self._input_doc_dir = output_dir_complement
    
    input_doc_dir = property(_get_input_doc_dir, _set_input_doc_dir)  
    
    
    # -- Property: schema_file
    
    def _get_schema_file(self):
        return self._schema_file
    
    def _set_schema_file(self, schema_file_cmpl):
        c_file = self.config_tree.xpath("file")[0]
        self._schema_file = self.structure_dir
        if self.source_type == 'amr':
            self._schema_file += c_file.get("amr_input_data_schema")
        if self.source_type == 'unl':
            self._schema_file += c_file.get("unl_input_data_schema")
        self._schema_file += schema_file_cmpl + ".ttl"
    
    schema_file = property(_get_schema_file, _set_schema_file) 
            
    
    # -- Property: source_sentence_files
    
    def _get_source_sentence_file(self):
        return self._source_sentence_file
    
    def _set_source_sentence_file(self, source_sentence_file_cmpl):
        self._source_sentence_file = self.input_doc_dir + self.source_corpus
        self._source_sentence_file += source_sentence_file_cmpl + '.ttl'
    
    source_sentence_file = property(_get_source_sentence_file, 
                                     _set_source_sentence_file) 
        
    
    #-------------------------------------------------
    # Methods to check the config
    #------------------------------------------------- 
    
    def is_base_ontology_produced_as_output(self):
        return self.target_ref in BASE_ONTOLOGY_REF
        
    
    #-------------------------------------------------
    # Methods
    #-------------------------------------------------      
            
    def get_main_config(self):
        """ Print the configuration """
        
        config_str = '\n' + '  *** Config (Main Parameters) *** '
        
        config_str += '\n' + '  -- Main Parameters'
        config_str += '\n  ----- config file: ' + self.config_file
        config_str += '\n  ----- uuid: ' + self.uuid_str
        config_str += '\n  ----- technical base name: ' + self.technical_base_name
        config_str += '\n  ----- source corpus: ' + self.source_corpus
        config_str += '\n  ----- target reference: ' + self.target_ref
        config_str += '\n  ----- process level: ' + self.process_level
        config_str += f'\n  ----- source type: {self.source_type}'
        config_str += f'\n  ----- extraction scheme: {self.extraction_scheme}'
        # config_str += '\n  ----- CTS reference (old): ' + self.cts_ref
        config_str += '\n  ----- output file: ' + self.output_file
        config_str += f'\n  ----- technical dir path: {self.technical_dir_path}'
        
        config_str += '\n' + '  *** - *** '
    
        return config_str
                
        
    def get_full_config(self):
        """ Print the configuration """
        
        config_str = '\n' + '  ***  Config (Full Parameters) *** '
        
        config_str += '\n  -- Base Parameters'
        config_str += '\n  ----- config file: ' + self.config_file
        config_str += '\n  ----- uuid: ' + self.uuid_str
        config_str += '\n  ----- technical base name: ' + self.technical_base_name
        config_str += '\n  ----- source corpus: ' + self.source_corpus
        config_str += '\n  ----- target reference: ' + self.target_ref
        config_str += '\n  ----- process level: ' + self.process_level
        config_str += f'\n  ----- source type: {self.source_type}'
        config_str += f'\n  ----- extraction scheme: {self.extraction_scheme}'
        
        # config_str += '\n' + '  -- Compositional Transduction Scheme (CTS)'
        # config_str += '\n  ----- CTS reference (old): ' + self.cts_ref
        
        config_str += '\n' + '  -- Directories'
        config_str += '\n  ----- base directory: ' + self.base_dir
        config_str += '\n  ----- structure directory: ' + self.structure_dir
        config_str += '\n  ----- CTS directory: ' + self.cts_dir
        config_str += '\n  ----- target frame directory: ' + self.target_frame_dir
        config_str += '\n  ----- input document directory: ' + self.input_doc_dir
        config_str += '\n  ----- base output dir: ' + self.base_output_dir
        config_str += '\n  ----- output directory: ' + self.output_dir
        config_str += '\n  ----- sentence output directory: ' + self.sentence_output_dir
        config_str += f'\n  ----- technical dir path: {self.technical_dir_path}'
        
        config_str += '\n' + '  -- Config File Definition'
        config_str += '\n  ----- schema file: ' + self.schema_file
        config_str += '\n  ----- semantic net file: ' + self.semantic_net_file
        config_str += '\n  ----- config param file: ' + self.config_param_file
        config_str += '\n  ----- base ontology file: ' + self.base_ontology_file
        config_str += '\n  ----- CTS file: ' + self.cts_file
        
        config_str += '\n' + '  -- Useful References for Ontology'
        config_str += '\n  ----- base URI: ' + self.base_uri
        config_str += '\n  ----- ontology suffix: ' + self.onto_suffix
        config_str += '\n  ----- ontology seed suffix: ' + self.onto_seed_suffix
        
        config_str += '\n' + '  -- Source File Definition'
        config_str += '\n  ----- source sentence file: ' + self.source_sentence_file
        
        config_str += '\n' + '  -- Target File Definition'
        config_str += '\n  ----- frame ontology file: ' + self.frame_ontology_file
        config_str += '\n  ----- frame ontology seed file: ' + self.frame_ontology_seed_file
        
        config_str += '\n' + '  -- Output'
        config_str += '\n  ----- ontology namespace: ' + self.output_ontology_namespace
        config_str += '\n  ----- output file: ' + self.output_file
        
        config_str += '\n' + '  *** - *** '
        
        return config_str