#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: New Tenet Extraction Engine (TEE)
#------------------------------------------------------------------------------
# Module, with a specific tenet extraction engine, to run the extraction 
# process by applying a list of transduction schemes (CTS) on a working 
# structure 
#==============================================================================

from rdflib import Graph
import os, sys, glob
import logging
from pathlib import Path

from importlib.machinery import SourceFileLoader
import importlib.util
import importlib

from utility.timer import timed, timer_return
from novTransduction.nov_rule import NovRule
from novTransduction.nov_sequence import NovSequence

# Logging
logger = logging.getLogger(__name__)

  
#==============================================================================
# Loading Functions
#==============================================================================

def load_cts(config):
    """ Load extraction scheme (CTS) from <extraction_scheme> file """
    
    try:
        cts_module = SourceFileLoader(config.extraction_scheme, 
                                      config.cts_file).load_module()
    
        return cts_module.rule_dir, cts_module.prefix_list, cts_module.scheme
    
    except FileNotFoundError:
        logger.error(' *** Error while loading scheme (load_cts) ***') 
        logger.debug(f' ----- current work directory: {os.getcwd()}')
        logger.debug(f' ----- cts_file unknown: {config.cts_file}') 
        
    except Exception:
        logger.error(' *** Error while loading scheme (load_cts) ***') 
        logger.debug(f' ----- current work directory: {os.getcwd()}')
        logger.debug(f' ----- extraction_scheme: {config.extraction_scheme}') 
        logger.debug(f' ----- cts_file: {config.cts_file}') 
    
    
def get_new_rule_set(rule_def_set, prefix_list):
    """ Get a set of new rules from <rule_def_set> (rule definition dictionary)  
    """
    
    try:
        rule_set = {}
        
        for rule_key, rule_def in rule_def_set.items():
            rule = NovRule()
            rule.load_dict(rule_def)
            rule.load_prefix_list(prefix_list)
            rule_set[rule_key] = rule
     
        return rule_set
    
    except:
        logger.error(' *** Error while loading rule set (get_new_rule_set) *** ') 
        logger.debug(f' ----- len(rule_def_set): {len(rule_def_set)}')
        logger.debug(f' ----- rule_key: {rule_key}') 
        logger.debug(f' ----- rule_def: {rule_def}')   
        logger.debug(f' ----- len(rule_set): {rule_set}') 

    
def load_rule_set(config, rule_dir, prefix_list):
    """ Load all rules into a set (as dictionary) from definition files  
        in <rule_dir> directory. 
    """
    
    try:
        rule_set = {}
        
        # -- Append rule paths to sys.path
        path_glob_pattern = config.cts_dir + rule_dir + '**/'
        for rule_path in glob.iglob(path_glob_pattern, recursive = True):
            sys.path.append(rule_path)
    
        # -- Load rule modules
        file_glob_pattern = config.cts_dir + rule_dir + '**/*.py'
        for rule_filename in glob.iglob(file_glob_pattern, recursive = True):
            
            if 'query_builder' not in rule_filename:
                               
                # -- old ---
                # spec = importlib.util.spec_from_file_location(rule_module_name,
                #                                               rule_filename)
                # rule_module = importlib.util.module_from_spec(spec)
                # sys.modules[rule_module_name] = rule_module
                # spec.loader.exec_module(rule_module)
                # -- ---
                
                # -- Import module
                rule_module_name = Path(rule_filename).stem
                rule_module = importlib.import_module(f'{rule_module_name}')
                
                # -- Update rule set
                if hasattr(rule_module, 'rule_set'):
                    rule_def_set = rule_module.rule_set
                    new_rule_set = get_new_rule_set(rule_def_set, prefix_list)
                    rule_set.update(new_rule_set)
    
        return rule_set
    
    except:
       logger.error(' *** Error while loading rule set (load_rule_set) *** ') 
       logger.debug(f' ----- path_glob_pattern: {path_glob_pattern}') 
       logger.debug(f' ----- sys.path: {sys.path}') 
       logger.debug(f' ----- file_glob_pattern: {file_glob_pattern}') 
       logger.debug(f' ----- rule_filename: {rule_filename}')  
       logger.debug(f' ----- rule_module_name: {rule_module_name}')
       logger.debug(f' ----- rule_module: {rule_module}')   
       logger.debug(f' ----- len(rule_def_set): {len(rule_def_set)}')
       logger.debug(f' ----- len(new_rule_set): {len(new_rule_set)}') 
       logger.debug(f' ----- len(rule_set): {len(rule_set)}')  
        

  
#==============================================================================
# Extraction step
#==============================================================================
        
def _prepare_sequence(sequence_def, rule_set):
    sequence = NovSequence()
    sequence.load_sequence_from_dict(sequence_def)
    sequence.load_rule_list(rule_set)
    return sequence


def _apply_refinement(graph, refinement_rule_list):
    """ Apply <refinement_rule_list> on <graph> """ 
    
    try:    
        all_new_triple_set = []
    
        for rule in refinement_rule_list:
            graph_length_before = len(graph)
            (graph, extracted_triple_set), exec_time_date = rule.apply(graph)
            all_new_triple_set.extend(extracted_triple_set)
            
            graph_length_after = len(graph)  
            new_triple_count = graph_length_after - graph_length_before
            message = "----- (refinement) {0}: {1} new triples ({2})"
            message = message.format(rule.label, 
                                     new_triple_count, 
                                     graph_length_after)
            if (new_triple_count > 0):
                logger.debug(message)
        
        return graph, all_new_triple_set
   
    except:
        logger.error(" *** Error while processing extraction (_apply_refinement) ***") 
        logger.debug(" ----- len(refinement_rule_list): {0} ".format(len(refinement_rule_list))) 
        logger.debug(" ----- last rule: {0} ".format(rule)) 
    
    
def _apply_old_sequence(graph, sequence, refinement_rule_list):
    """ Apply the rules of <sequence> on the working graph <graph> """ 
    
    try:
        logger.info("--- *** November Transduction *** Sequence: {0}".format(sequence.label))    
        all_new_triple_set = []
    
        for rule in sequence.rule_list:
            
            graph_length_before = len(graph)
            
            # -- apply rule
            (graph, extracted_triple_set), exec_time_date = rule.apply(graph)
            all_new_triple_set.extend(extracted_triple_set)
            
            new_triple_count = len(graph) - graph_length_before
            str = f"----- {rule.label}: "
            str += f"{new_triple_count}/{len(extracted_triple_set)} new triple"
            if new_triple_count > 1: str += f"s" 
            str += f" ({len(graph)}, {exec_time_date})"
            if (new_triple_count > 0):
                logger.info(str)
            else:
                logger.debug(str)
                
            # -- apply refinement
            graph, extracted_triple_set = _apply_refinement(graph, refinement_rule_list)
            all_new_triple_set.extend(extracted_triple_set)
    
        return graph, all_new_triple_set
   
    except Exception as e:
        logger.error(" *** Error while processing extraction (_apply_old_sequence) ***") 
        logger.debug(" ----- len(sequence): {0} ".format(len(sequence))) 
        logger.debug(" ----- last rule: {0} ".format(rule)) 
        logger.debug(" ----- last SPARQL query: \n{0} ".format(rule.get_query())) 
        logger.debug(" ----- len(extracted_triple_set): {0} ".format(len(extracted_triple_set))) 
        logger.debug(" ----- new_triple_count: {0} ".format(new_triple_count)) 
        
        
# @timer_return   
# def _apply_jan_sparql_query(graph, sparql_query):
#     extracted_triple_set = graph.query(sparql_query)
#     for triple in extracted_triple_set:
#         graph.add(triple)
#     return graph, extracted_triple_set

    
# def _apply_jan_rule_sequence(graph, rule_function):
#     """ Apply the (new) rule on the working graph <graph> """ 
    
#     try:
#         rule = rule_function()
#         logger.info(f"--- *** January Transduction *** Sequence: {rule.label}")    
#         all_new_triple_set = []
    
#         for (query_label, sparql_query) in rule.query_list:
            
#             graph_length_before = len(graph)
            
#             # -- apply rule
#             (graph, extracted_triple_set), exec_time_date = _apply_jan_sparql_query(graph, sparql_query) 
#             all_new_triple_set.extend(extracted_triple_set)
            
#             new_triple_count = len(graph) - graph_length_before
#             str = f"----- {query_label}: "
#             str += f"{new_triple_count}/{len(extracted_triple_set)} new triple"
#             if new_triple_count > 1: str += f"s" 
#             str += f" ({len(graph)}, {exec_time_date})"
#             if (new_triple_count > 0):
#                 logger.info(str)
#             else:
#                 logger.debug(str)
                
#             # -- apply refinement *** old ***
#             # graph, extracted_triple_set = _apply_refinement(graph, refinement_rule_list)
#             # all_new_triple_set.extend(extracted_triple_set)
    
#         return graph, all_new_triple_set
    
#     except AssertionError as ae:
#         logger.error(f' *** *** **** Assertion Error *** *** *** \n {ae}')  
        
#     except:
#         logger.error(f" *** Error while processing extraction (_apply_new_rule_sequence) ***")
#         logger.debug(f" ----- len(sequence): {len(rule.query_list)} ") 
#         logger.debug(f" ----- last rule: {query_label} ") 
#         logger.debug(f" ----- last SPARQL query: \n{sparql_query} ") 
#         logger.debug(f" ----- len(extracted_triple_set): {len(extracted_triple_set)} ") 
#         logger.debug(f" ----- new_triple_count: {new_triple_count} ") 
#         logger.debug(f" ----- exec_time_date: {exec_time_date} ") 
        
        
     
@timer_return   
def _apply_rule(graph, rule):
    rule_label, extracted_triple_set = rule(graph)
    for triple in extracted_triple_set:
        graph.add(triple)
    return rule_label, graph, extracted_triple_set

    
def _apply_sequence(graph, sequence_list):
    """ Apply the rule on the working graph <graph> """ 
    
    try:
        assert len(sequence_list) > 0, f'Houston, we have a problem: it is an empty sequence ({sequence_list})!'
        
        sequence_label = sequence_list[0]
        logger.info(f"--- Sequence: {sequence_label}")    
        all_new_triple_set = []
    
        for rule in sequence_list[1:]:
            
            graph_length_before = len(graph)
            
            # -- apply rule
            (rule_label, graph, extracted_triple_set), exec_time_date = _apply_rule(graph, rule) 
            all_new_triple_set.extend(extracted_triple_set)
            
            new_triple_count = len(graph) - graph_length_before
            str = f"----- {rule_label}: "
            str += f"{new_triple_count}/{len(extracted_triple_set)} new triple"
            if new_triple_count > 1: str += f"s" 
            str += f" ({len(graph)}, {exec_time_date})"
            if (new_triple_count > 0):
                logger.info(str)
            else:
                logger.debug(str)
                    
        return graph, all_new_triple_set
    
    except AssertionError as ae:
        logger.error(f' *** *** **** Assertion Error *** *** *** \n {ae}')  
        
    except:
        logger.error(f" *** Error while processing extraction (_apply_new_rule_sequence) ***")
        logger.debug(f" ----- len(sequence): {len(rule.query_list)} ") 
        logger.debug(f" ----- last rule: {query_label} ") 
        logger.debug(f" ----- last SPARQL query: \n{sparql_query} ") 
        logger.debug(f" ----- len(extracted_triple_set): {len(extracted_triple_set)} ") 
        logger.debug(f" ----- new_triple_count: {new_triple_count} ") 
        logger.debug(f" ----- exec_time_date: {exec_time_date} ") 
        
        
    
def _serialize_graph(config, graph, step_name):
    """ Serialize <graph> to a file """ 
    
    try:
        uuid_str = config.uuid_str
        work_file = config.output_file.replace('.ttl', '_' + step_name + '.ttl')
        base_ref = "http://{0}/{1}".format(uuid_str, step_name)
        
        message = "--- Serializing graph to {0} "
        message = message.format(Path(work_file).stem)
        logger.debug(message)
        logger.debug("----- step: {0}".format(step_name))
        logger.debug("----- id: {0}".format(uuid_str))
        logger.debug("----- work_file: {0}".format(work_file))
        logger.debug("----- base: {0}".format(base_ref))
        
        graph.serialize(destination=work_file, base=base_ref, format='turtle')

    except:
        logger.error(" *** Error while serializing graph (serialize_graph) ***") 
        logger.debug(" ----- work_file: {0} ".format(work_file)) 
        
     
def apply_step(config, graph, rule_set, step_number, step_name, step_sequence_def):
    """ Apply extraction step on the working graph """
    
    try:
        logger.info(f"-- Step {step_number}: {step_name}")
        
        # -- Initialize
        step_triple_list = []
        graph_length_before_step = len(graph)
        
        # -- Prepare refinement rule list
        refinement_sequence = _prepare_sequence(step_sequence_def[0], rule_set)
        refinement_rule_list = refinement_sequence.rule_list
        
        # -- Apply the sequences of the step
        for sequence_def in step_sequence_def[1:]:
            if (isinstance(sequence_def, dict)): # Old Rule Application
                sequence = _prepare_sequence(sequence_def, rule_set)
                graph, triple_list = _apply_old_sequence(graph, sequence, refinement_rule_list)
                step_triple_list.extend(triple_list)
            elif (isinstance(sequence_def, list)): # New Rule Application
                graph, triple_list = _apply_sequence(graph, sequence_def)
                step_triple_list.extend(triple_list)
            # else: # Old Jan Rule Application
            #     _apply_jan_rule_sequence(graph, sequence_def)
                                  
         
        # -- Serialize the working graph updated during the step
        if config.technical_dir_path is not None:
            os.makedirs(config.sentence_output_dir, exist_ok=True)
            _serialize_graph(config, graph, step_name)
        
        
        # -- Log extracted triple number
        str = "----- {0} triples extracted during {1} step"
        new_triple_count = len(graph) - graph_length_before_step
        logger.info(str.format(new_triple_count, step_name))
         
        return graph, step_triple_list
    
    except AssertionError:
        logger.error(f' *** *** **** Assertion Error *** *** *** \n')  
    
    except:
        logger.error(" *** Error while processing extraction (apply_step) ***") 
        logger.debug(f' ----- step_name = {step_name}') 
        logger.debug(f' ----- len(step_sequence_def) = {len(step_sequence_def)}') 
        logger.debug(f' ----- last sequence def = {sequence_def}') 
        logger.debug(f' ----- last sequence label = {sequence.label}') 
    
   
#==============================================================================
# Main Function
#==============================================================================   

def apply(config, graph):
    """ Apply extraction process on the working graph """
    
    try:
        
        # -- Loading Extraction Scheme
        logger.info(f"-- Loading Extraction Scheme ({config.extraction_scheme})")
        rule_dir, prefix_list, scheme = load_cts(config)
        logger.debug("----- Step number: {0}".format(len(scheme)))
            
        # -- Loading Extraction Rules
        logger.info("-- Loading Extraction Rules ({0}*)".format(rule_dir))
        rule_set = load_rule_set(config, rule_dir, prefix_list)
        logger.debug("----- Total rule number: {0}".format(len(rule_set)))
        
        # -- Apply each step of the scheme
        new_triple_list = []
        step_number = 0
        for step_name, step_sequence_def in scheme.items():
            step_number += 1
            graph, new_triple_list = apply_step(config, graph, rule_set,
                                                step_number, step_name, step_sequence_def)
    
        # -- Result: file containing only the factoids (last step result)
        if config.technical_dir_path is not None:
            os.makedirs(config.sentence_output_dir, exist_ok=True)
            factoid_file = config.output_file.replace('.ttl', '_factoid.ttl')
            logger.debug("--- Serializing graph to factoid file (" + factoid_file + ")")
            factoid_graph = Graph()
            for new_triple in new_triple_list:
                factoid_graph.add(new_triple) 
            logger.debug("----- Number of factoids: " + str(len(new_triple_list)))    
            base_ref = f'http://{config.uuid_str}/factoid'
            logger.debug("----- Graph base: " + base_ref) 
            factoid_graph.serialize(destination=factoid_file, 
                                    base=base_ref, 
                                    format='turtle')
                                
        return graph, new_triple_list   
    
    except:
        logger.error(' *** Error while processing extraction (apply) ***') 
        logger.debug(f' ----- config.extraction_scheme = {config.extraction_scheme}')  
        logger.debug(f' ----- rule_dir = {rule_dir}')  
        logger.debug(f' ----- scheme = {scheme}')      
        logger.debug(f' ----- step = {step_name}, {step_sequence_def}')  