#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: AMR CTR at 'OWL Generation' level
#------------------------------------------------------------------------------
# Module grouping compositional transduction rule_sets (CTR) for the analysis of 
# AMR structures, at 'OWL Generation' level
#==============================================================================


#==============================================================================
# CTR Set (CTR as Sparql querries)
#==============================================================================

rule_set = {}


# ---------------------------------------------
# OWL Generation: Preparation
# ---------------------------------------------

rule_set['compute-uri-for-owl-declaration-1'] = {
    'label': "compute-uri-for-owl-declaration-1",
    'comment': ("Compute URI for OWL declaration: ",
                " URI of classes "),
    'construction': """
       ?net1 net:hasClassURI ?newClassUri.
    """,
    'clause': """
        ?net1 a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassName ?className.
        FILTER NOT EXISTS { ?net1 net:deprecated true. }
        FILTER NOT EXISTS { ?net1 a net:Deprecated_Net. }
    """,
    'binding': """
    BIND (CONCAT(str(ext-out:), str(?className)) AS ?c1).
    BIND (uri(?c1) AS ?newClassUri).
    """
}

rule_set['compute-uri-for-owl-declaration-2'] = {
    'label': "compute-uri-for-owl-declaration-2",
    'comment': ("Compute URI for OWL declaration: ",
                " URI of mother classes "),
    'construction': """
       ?motherClassNet net:hasClassURI ?newClassUri.
    """,
    'clause': """
        ?net1 a [rdfs:subClassOf* net:Class_Net] ;
            net:hasMotherClassNet ?motherClassNet.
        ?motherClassNet a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassName ?motherClassName.
    """,
    'binding': """
    BIND (CONCAT(str(ext-out:), str(?motherClassName)) AS ?c1).
    BIND (uri(?c1) AS ?newClassUri).
    """
}

rule_set['compute-uri-for-owl-declaration-3'] = {
    'label': "compute-uri-for-owl-declaration-3",
    'comment': ("Compute URI for OWL declaration: ",
                " URI of individuals "),
    'construction': """
       ?net1 net:hasIndividualURI ?newIndividualUri.
    """,
    'clause': """
        ?net1 a net:Individual_Net ;
            net:hasIndividualLabel ?individualLabel.
        FILTER NOT EXISTS { ?net1 net:deprecated true. }
        FILTER NOT EXISTS { ?net1 a net:Deprecated_Net. }
    """,
    'binding': """
    BIND (REPLACE(?individualLabel, ' ', "-") AS ?iName1).
    BIND (LCASE(str(?iName1)) AS ?iName2).
    BIND (CONCAT(str(ext-out:), str(?iName2)) AS ?iName3).
    BIND (uri(?iName3) AS ?newIndividualUri).
    """
}

rule_set['compute-uri-for-owl-declaration-4'] = {
    'label': "compute-uri-for-owl-declaration-4",
    'comment': ("Compute URI for OWL declaration: ",
                " mother class URI of individuals ",
                " (case with a base ontology class) "),
    'construction': """
        ?net1 net:hasMotherClassURI ?newMotherClassUri.
    """,
    'clause': """
        ?net1 a net:Individual_Net ;
            net:hasBaseClassName ?motherClassName.
        FILTER NOT EXISTS { ?net1 net:deprecated true. }
        FILTER NOT EXISTS { ?net1 a net:Deprecated_Net. }
    """,
    'binding': """
    BIND (CONCAT(str(base-out:), str(?motherClassName)) AS ?c1).
    BIND (uri(?c1) AS ?newMotherClassUri).
    """
}

rule_set['compute-uri-for-owl-declaration-5'] = {
    'label': "compute-uri-for-owl-declaration-5",
    'comment': ("Compute URI for OWL declaration: ",
                " URI of properties "),
    'construction': """
       ?net1 net:hasPropertyURI ?newPropertyUri.
    """,
    'clause': """
        ?net1 a [rdfs:subClassOf* net:Property_Net] ;
            net:hasPropertyName ?propertyName.
        FILTER NOT EXISTS { ?net1 net:deprecated true. }
        FILTER NOT EXISTS { ?net1 a net:Deprecated_Net. }
    """,
    'binding': """
    BIND (CONCAT(str(ext-out:), str(?propertyName)) AS ?c1).
    BIND (uri(?c1) AS ?newPropertyUri).
    """
}

rule_set['compute-uri-for-owl-declaration-6'] = {
    'label': "compute-uri-for-owl-declaration-6",
    'comment': ("Compute URI for OWL declaration: ",
                " URI of properties (variant 12) "),
    'construction': """
       ?net1 net:hasProperty12URI ?newPropertyUri.
    """,
    'clause': """
        ?net1 a [rdfs:subClassOf* net:Property_Net] ;
            net:hasPropertyName12 ?propertyName12.
        FILTER NOT EXISTS { ?net1 net:deprecated true. }
        FILTER NOT EXISTS { ?net1 a net:Deprecated_Net. }
    """,
    'binding': """
    BIND (CONCAT(str(ext-out:), str(?propertyName12)) AS ?c1).
    BIND (uri(?c1) AS ?newPropertyUri).
    """
}

rule_set['compute-uri-for-owl-declaration-7'] = {
    'label': "compute-uri-for-owl-declaration-7",
    'comment': ("Compute URI for OWL declaration: ",
                " URI of mother properties "),
    'construction': """
        ?motherPropertyNet net:hasPropertyURI ?newPropertyUri.
    """,
    'clause': """
        ?net1 a [rdfs:subClassOf* net:Property_Net] ;
            net:hasMotherPropertyNet ?motherPropertyNet.
        ?motherPropertyNet a [rdfs:subClassOf* net:Property_Net] ;
            net:hasPropertyName ?motherPropertyName.
    """,
    'binding': """
    BIND (CONCAT(str(ext-out:), str(?motherPropertyName)) AS ?c1).
    BIND (uri(?c1) AS ?newPropertyUri).
    """
}

# *** New Net *** URI from Naming attribute 

# rule_set['compute-uri-for-owl-declaration-101'] = {
#     'label': "compute-uri-for-owl-declaration-101",
#     'comment': ("Compute URI for OWL declaration: ",
#                 " URI of properties (from Naming) "),
#     'construction': """
#        ?net1 net:hasPropertyURI ?newPropertyUri.
#     """,
#     'clause': """
#         ?net1 a [rdfs:subClassOf* net:Property_Net] ;
#             net:hasNaming ?propertyName.
#         FILTER NOT EXISTS { ?net1 net:deprecated true. }
#         FILTER NOT EXISTS { ?net1 a net:Deprecated_Net. }
#     """,
#     'binding': """
#     BIND (CONCAT(str(ext-out:), str(?propertyName)) AS ?c1).
#     BIND (uri(?c1) AS ?newPropertyUri).
#     """
# }

# rule_set['compute-uri-for-owl-declaration-102'] = {
#     'label': "compute-uri-for-owl-declaration-102",
#     'comment': ("Compute URI for OWL declaration: ",
#                 " URI of mother properties (from Naming) "),
#     'construction': """
#         ?motherPropertyNet net:hasPropertyURI ?newPropertyUri.
#     """,
#     'clause': """
#         ?net1 a [rdfs:subClassOf* net:Property_Net] ;
#             net:hasMotherPropertyNet ?motherPropertyNet.
#         ?motherPropertyNet a [rdfs:subClassOf* net:Property_Net] ;
#             net:hasNaming ?motherPropertyName.
#     """,
#     'binding': """
#     BIND (CONCAT(str(ext-out:), str(?motherPropertyName)) AS ?c1).
#     BIND (uri(?c1) AS ?newPropertyUri).
#     """
# }



# ---------------------------------------------
# OWL Generation: Atom Class
# ---------------------------------------------

rule_set['generate-atom-class'] = {
    'label': "generate-atom-class",
    'comment': "Generate Atom Class in the target ontology",
    'construction': """
       ?newClassUri rdf:type owl:Class ;
           rdfs:label ?className ;
           base-out:fromStructure ?sentenceRef.
    """,
    'clause': """
       ?net1 a net:Atom_Class_Net ;
           net:hasClassName ?className ;
           net:hasStructure ?sentenceRef ;
           net:hasClassURI ?newClassUri.
    """,
    'binding': """
    """
}

rule_set['classify-atom-class-1'] = {
    'label': "classify-atom-class-1",
    'comment': "Classify Atom Class in the target ontology",
    'construction': """
       ?newClassUri rdfs:subClassOf ?classType.
    """,
    'clause': """
       ?net1 a net:Atom_Class_Net ;
           net:hasClassType ?classType ;
           net:hasClassURI ?newClassUri.
    """,
    'binding': """
    """
}

rule_set['classify-atom-class-2'] = {
    'label': "classify-atom-class-2",
    'comment': "Classify Atom Class in the target ontology",
    'construction': """
       ?newClassUri rdfs:subClassOf base-out:Undetermined_Thing.
    """,
    'clause': """
       ?net1 a net:Atom_Class_Net ;
           net:hasClassURI ?newClassUri.
       FILTER NOT EXISTS { ?net1  net:hasClassType ?classType. }
       FILTER NOT EXISTS { ?newClassUri rdfs:subClassOf* base-out:Out_Structure. }
    """,
    'binding': """
    """
}




# ---------------------------------------------
# OWL Generation: Individual
# ---------------------------------------------

rule_set['generate-individual'] = {
    'label': "generate-individual",
    'comment': "Generate Individual in the target ontology",
    'construction': """
       ?newIndividualUri a owl:individual ;
           rdfs:label ?individualLabel ;
           base-out:fromStructure ?sentenceRef.
    """,
    'clause': """
       ?net1 a net:Individual_Net ;
           net:hasIndividualLabel ?individualLabel ;
           net:hasIndividualURI ?newIndividualUri.
        OPTIONAL { ?net1 net:hasStructure ?sentenceRef. }
    """,
    'binding': """
    """
}

rule_set['classify-individual-1'] = {
    'label': "classify-individual-1",
    'comment': "Classify Individual in the target ontology",
    'construction': """
       ?newIndividualUri a ?motherClassURI.
    """,
    'clause': """
        ?net1 a net:Individual_Net ;
           net:hasMotherClassURI ?motherClassURI ;
           net:hasIndividualURI ?newIndividualUri.
    """,
    'binding': """
    """
}

rule_set['classify-individual-2'] = {
    'label': "classify-individual-2",
    'comment': "Classify Individual in the target ontology",
    'construction': """
       ?newIndividualUri a ?motherClassURI.
    """,
    'clause': """
        ?net1 a net:Individual_Net ;
            net:hasMotherClassNet ?motherClassNet ;
            net:hasIndividualURI ?newIndividualUri.
        ?motherClassNet a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassURI ?motherClassURI.
        # -- old --- FILTER NOT EXISTS { ?motherClassNet net:deprecated true. }
    """,
    'binding': """
    """
}


# ---------------------------------------------
# OWL Generation: Atom Property
# ---------------------------------------------

rule_set['generate-atom-property-1'] = {
    'label': "generate-atom-property-1",
    'comment': "Generate Atom Property in the target ontology",
    'construction': """
       ?newPropertyUri a ?propertyType ;
           rdfs:subPropertyOf base-out:Out_ObjectProperty ;
           rdfs:label ?propertyName ;
           base-out:fromStructure ?sentenceRef.
    """,
    'clause': """
       ?net1 a net:Atom_Property_Net ;
           net:hasPropertyType ?propertyType ;
           net:hasPropertyName ?propertyName ;
           net:hasStructure ?sentenceRef ;
           net:hasPropertyURI ?newPropertyUri.
    """,
    'binding': """
    """
}

rule_set['generate-atom-property-12'] = {
    'label': "generate-atom-property-12",
    'comment': "Generate Atom Property (variant 12) in the target ontology",
    'construction': """
       ?newProperty12Uri a ?propertyType ;
           rdfs:subPropertyOf base-out:Out_ObjectProperty ;
           rdfs:label ?propertyName12 ;
           base-out:fromStructure ?sentenceRef.
    """,
    'clause': """
       ?net1 a net:Atom_Property_Net ;
           net:hasPropertyType ?propertyType ;
           net:hasPropertyName12 ?propertyName12 ;
           net:hasStructure ?sentenceRef ;
           net:hasProperty12URI ?newProperty12Uri.
    """,
    'binding': """
    """
}

rule_set['generate-inverse-relation'] = {
    'label': "generate-inverse-relation",
    'comment': ("Generate OWL inverseOf relation between two properties",
                " in the target ontology"),
    'construction': """
        ?prop1Uri owl:inverseOf ?prop2Uri.
    """,
    'clause': """
        ?net1 net:inverseOf ?net2.
        ?net1 net:hasPropertyURI ?prop1Uri.
        ?net2 net:hasPropertyURI ?prop2Uri.
    """,
    'binding': """
    """
}



# ---------------------------------------------
# OWL Generation: Composite Class (Taxonomy relation)
# ---------------------------------------------

rule_set['generate-composite-class'] = {
    'label': "generate-composite-class",
    'comment': "Generate Composite Class in the target ontology",
    'construction': """
        ?newClassUri a owl:Class ;
            rdfs:subClassOf ?motherClassUri ;
            rdfs:label ?className ;
            base-out:fromStructure ?sentenceRef.
    """,
    'clause': """
        ?net1 a net:Composite_Class_Net ;
            net:hasClassName ?className ;
            net:hasStructure ?sentenceRef ;
            net:hasClassURI ?newClassUri ;
            net:hasMotherClassNet ?motherClassNet.
        ?motherClassNet net:hasClassURI ?motherClassUri. 
    """,
    'binding': """
    """
}



# ---------------------------------------------
# OWL Generation: Class Restriction
# ---------------------------------------------

rule_set['add-restriction-to-class-1'] = {
    'label': "add-restriction-to-class-1",
    'comment': ("Add restriction to Class: ",
                " (1) on property, ",
                " (2) with some values from a class ",
                " (corresponding to a class net)",
                " (3) for standard case"),
    'construction': """
       ?newClassUri a owl:Class ;
           rdfs:subClassOf [
           a owl:Restriction ;
           owl:onProperty ?onPropertyUri ;
           owl:someValuesFrom ?valuesClassUri
           ] .
    """,
    'clause': """
        ?net1 a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassURI ?newClassUri ;
            net:hasRestriction ?restrictionNet. 
        ?restrictionNet a net:Restriction_Net;
            net:hasRestrictionOnProperty ?onPropertyNet ;
            net:hasRestrictionNetValue ?valueNet.
        ?onPropertyNet net:hasPropertyURI ?onPropertyUri. 
        ?valueNet a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassURI ?valuesClassUri. 
    """,
    'binding': """
    """
}

rule_set['add-restriction-to-class-2'] = {
    'label': "add-restriction-to-class-2",
    'comment': ("Add restriction to Class: ",
                " (1) on property, ",
                " (2) with individual for value ",
                " (corresponding to an individual net)",
                " (3) for standard case"),
    'construction': """
       ?newClassUri a owl:Class ;
           rdfs:subClassOf [
           a owl:Restriction ;
           owl:onProperty ?onPropertyUri ;
           owl:hasValue ?valueIndividualUri
           ] .
    """,
    'clause': """
       ?net1 a [rdfs:subClassOf* net:Class_Net] ;
           net:hasClassURI ?newClassUri ;
           net:hasRestriction ?restrictionNet. 
       ?restrictionNet a net:Restriction_Net;
           net:hasRestrictionOnProperty ?onPropertyNet ;
           net:hasRestrictionNetValue ?valueNet.
       ?onPropertyNet net:hasPropertyURI ?onPropertyUri. 
       ?valueNet a net:Individual_Net ;
           net:hasIndividualURI ?valueIndividualUri. 
    """,
    'binding': """
    """
}

rule_set['add-restriction-to-class-3'] = {
    'label': "add-restriction-to-class-3",
    'comment': ("Add restriction to Class: ",
                " (1) on property, ",
                " (2) with some values from a class ",
                " (corresponding to a class net)",
                " (3) for variant 01"),
    'construction': """
       ?newClassUri a owl:Class ;
           rdfs:subClassOf [
           a owl:Restriction ;
           owl:onProperty ?onPropertyUri ;
           owl:someValuesFrom ?valuesClassUri
           ] .
    """,
    'clause': """
        ?net1 a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassURI ?newClassUri ;
            net:hasRestriction01 ?restrictionNet. 
        ?restrictionNet a net:Restriction_Net;
            net:hasRestrictionOnProperty ?onPropertyNet ;
            net:hasRestrictionNetValue ?valueNet.
        ?onPropertyNet net:hasProperty12URI ?onPropertyUri. 
        ?valueNet a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassURI ?valuesClassUri. 
    """,
    'binding': """
    """
}

rule_set['add-restriction-to-class-4'] = {
    'label': "add-restriction-to-class-4",
    'comment': ("Add restriction to Class: ",
                " (1) on property, ",
                " (2) with individual for value ",
                " (corresponding to an individual net)",
                " (3) for variant 01"),
    'construction': """
       ?newClassUri a owl:Class ;
           rdfs:subClassOf [
           a owl:Restriction ;
           owl:onProperty ?onPropertyUri ;
           owl:hasValue ?valueIndividualUri
           ] .
    """,
    'clause': """
       ?net1 a [rdfs:subClassOf* net:Class_Net] ;
           net:hasClassURI ?newClassUri ;
           net:hasRestriction01 ?restrictionNet. 
       ?restrictionNet a net:Restriction_Net;
           net:hasRestrictionOnProperty ?onPropertyNet ;
           net:hasRestrictionNetValue ?valueNet.
       ?onPropertyNet net:hasProperty12URI ?onPropertyUri. 
       ?valueNet a net:Individual_Net ;
           net:hasIndividualURI ?valueIndividualUri. 
    """,
    'binding': """
    """
}

rule_set['add-restriction-to-class-5'] = {
    'label': "add-restriction-to-class-5",
    'comment': ("Add restriction to Class: ",
                " (1) on property, ",
                " (2) with some values from a class ",
                " (corresponding to a class net)",
                " (3) for variant 12"),
    'construction': """
       ?newClassUri a owl:Class ;
           rdfs:subClassOf [
           a owl:Restriction ;
           owl:onProperty ?onPropertyUri ;
           owl:someValuesFrom ?valuesClassUri
           ] .
    """,
    'clause': """
        ?net1 a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassURI ?newClassUri ;
            net:hasRestriction12 ?restrictionNet. 
        ?restrictionNet a net:Restriction_Net;
            net:hasRestrictionOnProperty ?onPropertyNet ;
            net:hasRestrictionNetValue ?valueNet.
        ?onPropertyNet net:hasProperty12URI ?onPropertyUri. 
        ?valueNet a [rdfs:subClassOf* net:Class_Net] ;
            net:hasClassURI ?valuesClassUri. 
    """,
    'binding': """
    """
}

rule_set['add-restriction-to-class-6'] = {
    'label': "add-restriction-to-class-6",
    'comment': ("Add restriction to Class: ",
                " (1) on property, ",
                " (2) with individual for value ",
                " (corresponding to an individual net)",
                " (3) for variant 12"),
    'construction': """
       ?newClassUri a owl:Class ;
           rdfs:subClassOf [
           a owl:Restriction ;
           owl:onProperty ?onPropertyUri ;
           owl:hasValue ?valueIndividualUri
           ] .
    """,
    'clause': """
       ?net1 a [rdfs:subClassOf* net:Class_Net] ;
           net:hasClassURI ?newClassUri ;
           net:hasRestriction12 ?restrictionNet. 
       ?restrictionNet a net:Restriction_Net;
           net:hasRestrictionOnProperty ?onPropertyNet ;
           net:hasRestrictionNetValue ?valueNet.
       ?onPropertyNet net:hasProperty12URI ?onPropertyUri. 
       ?valueNet a net:Individual_Net ;
           net:hasIndividualURI ?valueIndividualUri. 
    """,
    'binding': """
    """
}



# ---------------------------------------------
# OWL Generation: Composite Property
# ---------------------------------------------

# rule_set['generate-composite-property-1'] = {
#     'label': "generate-composite-property-1",
#     'comment': "Generate Composite Property in the target ontology",
#     'construction': """
#        ?newPropertyUri a ?propertyType ;
#            rdfs:label ?propertyName ;
#            base-out:fromStructure ?sentenceRef.
#     """,
#     'clause': """
#        ?net1 a net:Composite_Property_Net ;
#            net:hasPropertyType ?propertyType ;
#            net:hasPropertyName ?propertyName ;
#            net:hasStructure ?sentenceRef ;
#            net:hasPropertyURI ?newPropertyUri. 
#     """,
#     'binding': """
#     """
# }

# rule_set['generate-composite-property-2'] = {
#     'label': "generate-composite-property-2",
#     'comment': "Generate Composite Property in the target ontology",
#     'construction': """
#        ?newPropertyUri a ?propertyType ;
#            rdfs:subPropertyOf base-out:Out_ObjectProperty ;
#            rdfs:label ?propertyName ;
#            base-out:fromStructure ?sentenceRef.
#     """,
#     'clause': """
#        ?net1 a net:Composite_Property_Net ;
#            net:hasPropertyType ?propertyType ;
#            net:hasNaming ?propertyName ;
#            net:hasStructure ?sentenceRef ;
#            net:hasPropertyURI ?newPropertyUri. 
#     """,
#     'binding': """
#     """
# }

# rule_set['generate-composite-property-3'] = {
#     'label': "generate-composite-property-3",
#     'comment': "Generate Composite Property in the target ontology",
#     'construction': """
#        ?newPropertyUri rdfs:subPropertyOf ?motherPropertyUri.
#     """,
#     'clause': """
#        ?net1 a net:Composite_Property_Net ;
#            net:hasPropertyURI ?newPropertyUri ;
#            net:hasMotherPropertyNet ?motherPropertyNet.
#        ?motherPropertyNet net:hasPropertyURI ?motherPropertyUri. 
#     """,
#     'binding': """
#     """
# }
