#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to conjunctive phenomena or (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to analyse conjunctive phenomena (or)
# Rule: property(class, or_phenomena) => compositeClass
#==============================================================================

import rdflib
from rdflib import Graph
from rdflib.namespace import FOAF, RDF

import transduction
from transduction import net
from transduction.rdfterm_computer import produce_uriref, produce_literal
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_axiom_naming
from transduction.naming_computer import define_composite_naming_2


#==============================================================================
# Select Pattern: Atom / Composite Property Net
#==============================================================================  
       
def __search_pattern_1(graph):
    select_data_list = ['?property_net']
    clause_list = [(f'?property_net a net:Atom_Property_Net.')]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    rule_pattern_set = graph.query(query_code) 
    return rule_pattern_set


def __search_pattern_2(graph):
    select_data_list = ['?property_net']
    clause_list = [(f'?property_net a net:Composite_Property_Net.')]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    rule_pattern_set = graph.query(query_code) 
    return rule_pattern_set
 
    
def __get_atom_property_net_list(graph):
    
    # -- Atom Property List
    atom_property_net_list = []
    atom_property_set = __search_pattern_1(graph)
    for selection in atom_property_set:
        property_net = net.AtomPropertyNet(graph, uri=selection.property_net)
        atom_property_net_list.append(property_net)
    
    return atom_property_net_list
 
    
def __get_composite_property_net_list(graph):
    
    # -- Composite Property List
    composite_property_net_list = []
    composite_property_set = __search_pattern_2(graph)
    for selection in composite_property_set:
        property_net = net.CompositePropertyNet(graph, uri=selection.property_net)
        composite_property_net_list.append(property_net)
    
    return composite_property_net_list
 
    
def __get_mother_property_net_list(atom_property_net_list, composite_property_net_list):
        
    # -- URI List
    mother_property_uri_list = []
    for net in composite_property_net_list:
        if net.mother_property_net is not None:
            mother_property_uri_list += net.mother_property_net
      
    # -- Net List
    mother_property_net_list = []
    for net in (atom_property_net_list + composite_property_net_list):
        if net.uri in mother_property_uri_list:
            mother_property_net_list.append(net)
    
    return mother_property_net_list



#==============================================================================
# Check Methods
#==============================================================================

def __is_property_to_generate(property_net):
    check_1 = property_net.is_deprecated()
    if is_instance(property_net.mother_property_net, list):
        check_2 = len(property_net.mother_property_net) > 0
    else:
        check_2 = False
    return not check_1 or check_2
    
   
    
#==============================================================================
# Generator Methods
#==============================================================================

def __compute_property_uri(property_net):
    if isinstance(property_net.property_uri, list) and len(property_net.property_uri) > 0:
        property_uri = property_net.property_uri[0]
    else: # computation of property_uri
        net_naming = property_net.get_attribute_first_value(property_net.naming)
        property_uri = produce_uriref(property_net.support_graph, f'ext-out:{net_naming}')
    return property_uri


def __get_mother_property_uri(graph, mother_property_net_uri):
    mother_property_net = net.PropertyNet(graph, uri=mother_property_net_uri)
    return __compute_property_uri(mother_property_net)


def __define_triple(new_property_uri, relation, data):    
    triple_list = []    
    for predicat in data:
        triple_list.append((new_property_uri, relation, predicat))        
    return triple_list


def __generate_owl_taxonomic_relation(graph, new_property_uri, net):
    
    triple_list = []
    relation = produce_uriref(graph, 'rdfs:subPropertyOf')
    predicat = None
    
    if net.type_id == 'Atom_Property_Net':
        predicat = produce_uriref(graph, 'base-out:Out_ObjectProperty')
        
    if net.type_id == 'Composite_Property_Net':
        mother_property_net_uri = net.get_attribute_first_value(net.mother_property_net)
        if mother_property_net_uri is None:
            predicat = produce_uriref(graph, 'base-out:Out_ObjectProperty')
        else :
            predicat = __get_mother_property_uri(graph, mother_property_net_uri)
    
    if predicat is not None:
        triple_list.append((new_property_uri, relation, predicat))
        
    return triple_list


def __generate_owl_triple_definition(graph, net):
    
    triple_list = []
             
    new_property_uri = __compute_property_uri(net)
    
    rdf_type_uri = produce_uriref(graph, RDF.type)
    triple_list += __define_triple(new_property_uri, rdf_type_uri, net.property_type)
    
    relation = produce_uriref(graph, 'rdfs:label')
    triple_list += __define_triple(new_property_uri, relation, net.property_name)
    
    relation = produce_uriref(graph, 'base-out:fromStructure')
    triple_list += __define_triple(new_property_uri, relation, net.structure)
    
    triple_list += __generate_owl_taxonomic_relation(graph, new_property_uri, net)
    
    return triple_list

  
    
#==============================================================================
# Main Method: analyze_phenomena_or_1
#==============================================================================

def generate_owl_property(graph):
    
    # -- Rule Initialization 
    rule_label = 'generate OWL property' 
    rule_triple_list = []

    # -- Get property net listings
    atom_property_net_list =  __get_atom_property_net_list(graph)
    composite_property_net_list =  __get_composite_property_net_list(graph)
    mother_property_net_list =  __get_mother_property_net_list(atom_property_net_list, 
                                                               composite_property_net_list)   
    
    # -- Triple Definition for 'not deprecated property net'
    for property_net in (atom_property_net_list + composite_property_net_list):
        if not property_net.is_deprecated():
            rule_triple_list += __generate_owl_triple_definition(graph, property_net)
     
    # -- Triple Definition for 'mother property net'
    for property_net in mother_property_net_list:
        rule_triple_list += __generate_owl_triple_definition(graph, property_net)
        
    return rule_label, rule_triple_list