#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to generate OWL class
#------------------------------------------------------------------------------
# Generative rule to build OWL classes
# Rule: classNet => OWL class
#==============================================================================

import rdflib
from rdflib import Graph, Namespace, BNode
from rdflib.namespace import FOAF, RDF, RDFS, OWL

import transduction
from transduction import net
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )

from transduction.query_builder import (
    generate_select_query,
    generate_ask_query
)

DEFAULT_CLASS_TYPE = 'base-out:Undetermined_Thing'


#==============================================================================
# Methods to search for patterns: Atom / Composite Class Net
#==============================================================================  
       
def __search_pattern_1(graph):
    
    # SPARQL query construction
    select_data_list = ['?class_net']
    clause_list = [(f'?class_net a net:Atom_Class_Net.')]
    query_code = generate_select_query(graph, select_data_list, clause_list)

    # SPARQL query execution
    rule_pattern_set = graph.query(query_code) 

    # Result (pattern set)
    return rule_pattern_set


def __search_pattern_2(graph):
    
    # SPARQL query construction
    select_data_list = ['?class_net']
    clause_list = [(f'?class_net a net:Composite_Class_Net.')]
    query_code = generate_select_query(graph, select_data_list, clause_list)

    # SPARQL query execution
    rule_pattern_set = graph.query(query_code) 

    # Result (pattern set)
    return rule_pattern_set



#==============================================================================
# Methods to ask about net (using SPARQL ASK query)
#==============================================================================

def is_class_net(graph, net_uri):
    
    # SPARQL query construction
    clause_list = ['?classUri a [rdfs:subClassOf* net:Class_Net].']
    query_code = generate_ask_query(graph, clause_list)

    # SPARQL query execution
    result = graph.query(query_code, initBindings={'classUri': net_uri})

    # Result (True or False)
    return query_code, bool(result)


def is_individual_net(graph, net_uri):
    
    # SPARQL query construction
    clause_list = ['?classUri a [rdfs:subClassOf* net:Individual_Net].']
    query_code = generate_ask_query(graph, clause_list)

    # SPARQL query execution
    result = graph.query(query_code, initBindings={'classUri': net_uri})

    # Result (True or False)
    return query_code, bool(result) 



#==============================================================================
# Methods to get useful net list 
#==============================================================================
    
def __get_atom_class_net_list(graph):
    
    # -- Atom Class List
    atom_class_net_list = []
    atom_class_set = __search_pattern_1(graph)
    for selection in atom_class_set:
        class_net = net.AtomClassNet(graph, uri=selection.class_net)
        atom_class_net_list.append(class_net)
    
    return atom_class_net_list
 
    
def __get_composite_class_net_list(graph):
    
    # -- Composite Class List
    composite_class_net_list = []
    composite_class_set = __search_pattern_2(graph)
    for selection in composite_class_set:
        class_net = net.CompositeClassNet(graph, uri=selection.class_net)
        composite_class_net_list.append(class_net)
    
    return composite_class_net_list
 
    
def __get_mother_class_net_list(atom_class_net_list, composite_class_net_list):
        
    # -- URI List
    mother_class_uri_list = []
    for net in composite_class_net_list:
        if net.mother_class_net is not None:
            mother_class_uri_list += net.mother_class_net
      
    # -- Net List
    mother_class_net_list = []
    for net in (atom_class_net_list + composite_class_net_list):
        if net.uri in mother_class_uri_list:
            mother_class_net_list.append(net)
    
    return mother_class_net_list



#==============================================================================
# Check Method(s)
#==============================================================================

def __is_class_to_generate(class_net):
    check_1 = class_net.is_deprecated()
    if is_instance(class_net.mother_class_net, list):
        check_2 = len(class_net.mother_class_net) > 0
    else:
        check_2 = False
    return not check_1 or check_2
    
   
    
#==============================================================================
# Generator Methods
#==============================================================================

# -- Base Methods to compute URI
# ------------------------------------------

def __compute_class_uri(class_net):
    if isinstance(class_net.class_uri, list) and len(class_net.class_uri) > 0:
        class_uri = class_net.class_uri[0]
    else: # computation of class_uri
        net_naming = class_net.get_attribute_first_value(class_net.naming)
        class_uri = produce_uriref(class_net.support_graph, f'ext-out:{net_naming}')
    return class_uri


def __get_mother_class_uri(graph, mother_class_net_uri):
    mother_class_net = net.ClassNet(graph, uri=mother_class_net_uri)
    return __compute_class_uri(mother_class_net)


def __compute_class_type_uri(class_net):
    if isinstance(class_net.class_type, list) and len(class_net.class_type) > 0:
        class_type = class_net.class_type[0]
    else: # default class_type
        class_type = f'{DEFAULT_CLASS_TYPE}'
    return produce_uriref(class_net.support_graph, class_type)


def __compute_property_uri(property_net):
    if isinstance(property_net.property_uri, list) and len(property_net.property_uri) > 0:
        property_uri = property_net.property_uri[0]
    else: # computation of class_uri
        net_naming = property_net.get_attribute_first_value(property_net.naming)
        property_uri = produce_uriref(property_net.support_graph, f'ext-out:{net_naming}')
    return property_uri



# -- Base Method to define triple
# --------------------------------

def __define_triple(new_class_uri, relation, data):    
    triple_list = []    
    for predicat in data:
        triple_list.append((new_class_uri, relation, predicat))        
    return triple_list



# -- Base Methods to generate restriction
# ----------------------------------------

def __generate_restriction_with_some_values(
        graph, class_uri, on_property_uri, values_class_uri):
    
    # -- namespace = Namespace("https://tenet.tetras-libre.fr/semantic-net#")
    restriction = BNode()

    triple_list = [
        (class_uri, RDFS.subClassOf, restriction),
        (restriction, RDF.type, OWL.Restriction),
        (restriction, OWL.onProperty, on_property_uri),
        (restriction, OWL.someValuesFrom, values_class_uri)
    ]
    
    return triple_list


def __generate_restriction_with_specific_value(
        graph, class_uri, on_property_uri, value_individual_uri):
    
    # -- namespace = Namespace("https://tenet.tetras-libre.fr/semantic-net#")
    restriction = BNode()

    triple_list = [
        (class_uri, RDFS.subClassOf, restriction),
        (restriction, RDF.type, OWL.Restriction),
        (restriction, OWL.onProperty, on_property_uri),
        (restriction, OWL.hasValue, value_individual_uri)
    ]
    
    return triple_list


# -- Methods to generate OWL triple
# -----------------------------------

def __generate_owl_taxonomic_relation(graph, new_class_uri, net):
    
    triple_list = []
    relation = produce_uriref(graph, 'rdfs:subClassOf')
    predicat = None
        
    if net.type_id == 'Atom_Class_Net':
        predicat = __compute_class_type_uri(net)
        
    if net.type_id == 'Composite_Class_Net':
        mother_class_net_uri = net.get_attribute_first_value(net.mother_class_net)
        if mother_class_net_uri is None:
            predicat = __compute_class_type_uri(net)
        else :
            predicat = __get_mother_class_uri(graph, mother_class_net_uri)
    
    if predicat is not None:
        triple_list.append((new_class_uri, relation, predicat))
        
    return triple_list


def __generate_logical_restriction(graph, new_class_uri, restriction_net_uri):
    
    triple_list = []
    
    restriction_net = net.RestrictionNet(graph, uri=restriction_net_uri)
        
    if (restriction_net.has_restriction_property 
        and restriction_net.has_restriction_net_value):
        
        # OnProperty URI
        property_net_uri = restriction_net.restriction_property[0]
        property_net = net.PropertyNet(graph, uri=property_net_uri)
        on_property_uri = __compute_property_uri(property_net)
        
        # ValueNet URI
        value_net_uri = restriction_net.restriction_net_value[0]
        
        # Restriction generation depending of value net type
        if (is_class_net(graph, value_net_uri)): # Case 1: Class Net
            
            value_net = net.ClassNet(graph, uri=value_net_uri) 
            values_class_uri = __compute_class_uri(value_net)
            
            triple_list += __generate_restriction_with_some_values(
                graph, new_class_uri, on_property_uri, values_class_uri)
            
        elif (is_individual_net(graph, value_net_uri)): # Case 2: Individual Net
            
            value_net = net.IndividualNet(graph, uri=value_net_uri) 
            value_individual_uri = __compute_class_uri(value_net)
            
            triple_list += __generate_restriction_with_specific_value(
                graph, new_class_uri, on_property_uri, value_individual_uri)

    return triple_list




# -- Main Generation Methods
# --------------------------------

def __generate_owl_triple_definition(graph, net):
    
    triple_list = []
             
    new_class_uri = __compute_class_uri(net)
    
    rdf_type_uri = produce_uriref(graph, RDF.type)
    class_type_list = [produce_uriref(graph, 'owl:Class')]
    triple_list += __define_triple(new_class_uri, rdf_type_uri, class_type_list)
    
    relation = produce_uriref(graph, 'rdfs:label')
    triple_list += __define_triple(new_class_uri, relation, net.class_name)
    
    relation = produce_uriref(graph, 'base-out:fromStructure')
    triple_list += __define_triple(new_class_uri, relation, net.structure)
    
    triple_list += __generate_owl_taxonomic_relation(graph, new_class_uri, net)
    
    if net.type_id == 'Composite_Class_Net':
        for restriction_net in net.restriction:
            triple_list += __generate_logical_restriction(graph, new_class_uri, restriction_net)
    
    return triple_list



# def __generate_owl_triple_definition_of_simple_class(graph, class_net):
#     triple_list = __generate_owl_triple_definition(graph, class_net)
#     return triple_list


# def __generate_owl_triple_definition_of_composite_class(graph, class_net):
#     triple_list = __generate_owl_triple_definition(graph, class_net)

#     # Add logical restrictions
#     triple_list += __generate_logical_restrictions(graph, class_net)

#     return triple_list


  
    
#==============================================================================
# Main Method: generate_owl_class
#==============================================================================

def generate_owl_class(graph):
    
    # -- Rule Initialization 
    rule_label = 'generate OWL class' 
    rule_triple_list = []

    # -- Get class net listings
    atom_class_net_list = __get_atom_class_net_list(graph)
    composite_class_net_list = __get_composite_class_net_list(graph)
    mother_class_net_list = __get_mother_class_net_list(atom_class_net_list, composite_class_net_list)   

    # Génération des triplets pour les classes atomiques
    for class_net in atom_class_net_list:
        if not class_net.is_deprecated():
            triple_list = __generate_owl_triple_definition(graph, class_net)
            rule_triple_list += triple_list

    # Génération des triplets pour les classes composites
    for class_net in composite_class_net_list:
        if not class_net.is_deprecated():
            triple_list = __generate_owl_triple_definition(graph, class_net)
            rule_triple_list += triple_list

    # Génération des triplets pour les classes mères
    for class_net in mother_class_net_list:
            triple_list = __generate_owl_triple_definition(graph, class_net)
            rule_triple_list += triple_list

    return rule_label, rule_triple_list