#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Semantic Net Class Test
#------------------------------------------------------------------------------
# Script to test the semantic net classes
#==============================================================================

import subprocess, os
import rdflib  
from rdflib import Graph, Namespace
from rdflib.namespace import NamespaceManager

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph1'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet

from tenet.transduction.net import Net

from tenet.transduction.net import ClassNet
from tenet.transduction.net import AtomClassNet
from tenet.transduction.net import CompositeClassNet

from tenet.transduction.net import PropertyNet
from tenet.transduction.net import AtomPropertyNet
from tenet.transduction.net import CompositePropertyNet

from tenet.transduction.net import IndividualNet

from tenet.transduction.net import ValueNet

from tenet.transduction.net import PhenomenaNet

from tenet.transduction.net import RestrictionNet

from tenet.transduction.net import AxiomNet



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    graph.bind('amr', Namespace('https://amr.tetras-libre.fr/rdf/schema#'))
    graph.bind('net', Namespace('https://tenet.tetras-libre.fr/semantic-net#'))
    graph.bind('base-out', Namespace('https://tenet.tetras-libre.fr/base-ontology#'))
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph
 
    
def print_net_attributes(net):
    print(f' ----- Net URI: {net.uri}')
    print(f'\n *** Net attributes ({net.type_id})) ***')
    for attr in net.attr_list:
        net_attr_ref = f'net.{attr}'
        print(f' ----- {attr}: {eval(net_attr_ref)}') 


#==============================================================================
# Development Test
#==============================================================================

def test_set_attribute_net(graph, net_uri):
    
    print('\n -- Net')
    net = Net(graph, uri=net_uri)
    print('\n /// before setting')
    print_net_attributes(net)
    
    net.base_node = 'amr:node_a'
    net.node_list = ['amr:node_a', 'amr:node_b']
    net.structure = ' *** test-structure ***'
    net.naming = ' *** test-naming ***'
    print('\n /// after setting (1)')
    print_net_attributes(net)
    
    assert isinstance(net.base_node[0], rdflib.term.URIRef), f'Houston, we have a problem: result base_node is not an URIRef'
    
    net.node_list = ['amr:node_a', 'amr:node_b']
    print('\n /// after setting (2)')
    print_net_attributes(net)
    
    net.node_list = []
    print('\n /// after setting (3)')
    print_net_attributes(net)


def test_set_attribute_class_net(graph, net_uri):
    print('\n -- Class Net')
    net = ClassNet(graph, uri=net_uri)
    net.class_name = ' *** test-class-name (1) ***'
    print_net_attributes(net)
  

def test_set_attribute_atom_class_net(graph, net_uri):  
    print('\n -- Atom Class Net')
    net = AtomClassNet(graph, uri=net_uri)
    net.class_name = ' *** test-class-name (2) ***'
    print_net_attributes(net)
   

def test_set_attribute_composite_class_net(graph, net_uri): 
    print('\n -- Composite Class Net (1)')
    net = CompositeClassNet(graph, uri=net_uri)
    net.mother_class_net = 'net:mother-test'
    net.restriction = 'net:restriction-test'
    net.restriction01 = 'net:restriction-test-01'
    print_net_attributes(net)
    

def test_set_attribute_property_net(graph, net_uri): 
    print('\n -- Property Net')
    net = PropertyNet(graph, uri=net_uri)
    net.property_name = ' *** test-value ***'
    print_net_attributes(net)
    

def test_set_attribute_atom_property_net(graph, net_uri): 
    print('\n -- Atom Property Net')
    net = AtomPropertyNet(graph, uri=net_uri)
    net.property_name = ' *** test-value ***'
    net.core_role = ' *** test-value ***'
    net.target_argument_node = 'amr:node_a'
    net.property_type = 'net:type-test'
    net.property_name01 = ' *** test-value ***'
    net.property_name10 = ' *** test-value ***'
    net.property_name12 = ' *** test-value ***'
    print_net_attributes(net)
    

def test_set_attribute_composite_property_net(graph, net_uri): 
    print('\n -- Composite Property Net')
    net = CompositePropertyNet(graph, uri=net_uri)
    net.property_name = ' *** test-value ***'
    net.core_role = 'True'
    net.target_argument_node = 'amr:node_a'
    net.property_type = 'net:prop-type'
    net.restriction = 'net:restriction-test'
    print_net_attributes(net)
    

def test_set_attribute_individual_net(graph, net_uri): 
    print('\n -- Individual Net')
    net = IndividualNet(graph, uri=net_uri)
    net.base_class_name = ' *** test-value ***'
    net.mother_class_net = 'net:mother-test'
    net.individual_label = ' *** test-value ***'
    print_net_attributes(net)
    

def test_set_attribute_value_net(graph, net_uri): 
    print('\n -- Value Net')
    net = ValueNet(graph, uri=net_uri)
    net.value_label = ' *** test-value ***'
    print_net_attributes(net)
       

def test_set_attribute_phenomena_net(graph, net_uri): 
    print('\n -- Phenomena Net (1)')
    net = PhenomenaNet(graph, uri=net_uri)
    net.phenomena_type = 'net:phenomena-type-test'
    net.phenomena_ref = ' *** test-value ***'
    print_net_attributes(net)
    

def test_set_attribute_restriction_net(graph, net_uri): 
    print('\n -- Restriction Net')
    net = RestrictionNet(graph, uri=net_uri)
    net.target_node = 'amr:node_a'
    net.restriction_property = 'net:restriction-property-test'
    net.restriction_net_value = 'net:restriction-value-test'
    print_net_attributes(net)
    

def test_set_attribute_axiom_net(graph, net_uri): 
    print('\n -- Restriction Net')
    net = AxiomNet(graph, uri=net_uri)
    net.axiom_name = 'disjointProperty'
    net.axiom_uri = 'owl:propertyDisjointWith'
    net.axiom_net_argument = ['net:atomProperty_direct_d2', 'net:atomProperty_not-direct_d2']
    print_net_attributes(net)



#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph = load_test_graph()
    print('\n \n')
    
    print('\n *** Development Test ***')
    
    test_set_attribute_net(graph, 'net:atomClass_sun_s2')

    test_set_attribute_class_net(graph, 'net:atomClass_sun_s2')
    test_set_attribute_atom_class_net(graph, 'net:atomClass_sun_s2')
    test_set_attribute_composite_class_net(graph, 'net:compositeClass_orbit-hasManner-conjunction-OR_o2')
    test_set_attribute_composite_class_net(graph, 'net:compositeClass_system-hasPart-sun-and-object_s')

    test_set_attribute_property_net(graph, 'net:atomProperty_orbit_o2')     
    test_set_attribute_atom_property_net(graph, 'net:atomProperty_orbit_o2')
    test_set_attribute_composite_property_net(graph, 'net:compositeProperty_not-direct_d2')
    
    test_set_attribute_individual_net(graph, 'net:individual_system_SolarSystem')  

    test_set_attribute_value_net(graph, 'net:value_SolarSystem_blankNode')  
     
    test_set_attribute_phenomena_net(graph, 'net:phenomena_conjunction-AND_a')
    test_set_attribute_phenomena_net(graph, 'net:phenomena_conjunction-OR_o3') 

    test_set_attribute_restriction_net(graph, 'net:restriction_hasPart_sun')  

    test_set_attribute_axiom_net(graph, 'net:axiom_disjointProperties_direct_not-direct')                      
    
    print('\n \n')